"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const selectors_1 = require("../styles/selectors");
const selectors_2 = require("../styles/utils/selectors");
const query_1 = require("../styles/selectors/query");
const context_1 = require("../styles/context");
const utils_1 = require("../utils/utils");
const options_1 = require("../options");
const style_1 = require("../styles/context/style");
function getScopedSelectors(style) {
    const resolvedSelectors = selectors_1.getResolvedSelectors(style);
    return resolvedSelectors.map(getScopedSelector).filter(utils_1.isDefined);
}
function getScopedSelector(resolvedSelector) {
    const { selector } = resolvedSelector;
    const specialNodeIndex = selector.findIndex((s) => selectors_2.isDeepCombinator(s) || selectors_2.isVueSpecialPseudo(s));
    if (specialNodeIndex >= 0) {
        const specialNode = selector[specialNodeIndex];
        if (selectors_2.isDeepCombinator(specialNode) || selectors_2.isVDeepPseudo(specialNode)) {
            const scopedCandidateSelector = selector.slice(0, specialNodeIndex);
            const last = scopedCandidateSelector.pop();
            if (last && !selectors_2.isDescendantCombinator(last)) {
                scopedCandidateSelector.push(last);
            }
            return scopedCandidateSelector;
        }
        else if (selectors_2.isVSlottedPseudo(specialNode)) {
            return selector.slice(0, specialNodeIndex + 1);
        }
        else if (selectors_2.isVGlobalPseudo(specialNode)) {
            return null;
        }
        return [...selector];
    }
    return [...selector];
}
module.exports = {
    meta: {
        docs: {
            description: "disallow selectors defined that is not used inside `<template>`",
            categories: [],
            default: "warn",
            url: "https://future-architect.github.io/eslint-plugin-vue-scoped-css/rules/require-selector-used-inside.html",
        },
        fixable: null,
        messages: {
            unused: "The selector `{{selector}}` is unused in the template.",
        },
        schema: [
            {
                type: "object",
                properties: {
                    ignoreBEMModifier: {
                        type: "boolean",
                    },
                    captureClassesFromDoc: {
                        type: "array",
                        items: [
                            {
                                type: "string",
                            },
                        ],
                        minItems: 0,
                        uniqueItems: true,
                    },
                },
                additionalProperties: false,
            },
        ],
        type: "suggestion",
    },
    create(context) {
        if (!utils_1.hasTemplateBlock(context)) {
            return {};
        }
        const styles = context_1.getStyleContexts(context)
            .filter(style_1.isValidStyleContext)
            .filter((style) => style.scoped);
        if (!styles.length) {
            return {};
        }
        const reporter = context_1.getCommentDirectivesReporter(context);
        const reportedSet = new Set();
        function report(nodes) {
            const last = nodes[nodes.length - 1];
            if (!reportedSet.has(last)) {
                reporter.report({
                    loc: {
                        start: nodes[0].loc.start,
                        end: last.loc.end,
                    },
                    messageId: "unused",
                    data: {
                        selector: nodes.map((n) => n.selector).join(""),
                    },
                });
                reportedSet.add(last);
            }
        }
        function verifySelector(queryContext, scopedSelector) {
            let targetsQueryContext = queryContext;
            const selectorNodes = scopedSelector
                .filter((s) => selectors_2.isSelectorCombinator(s) ||
                selectors_2.isTypeSelector(s) ||
                selectors_2.isIDSelector(s) ||
                selectors_2.isClassSelector(s) ||
                selectors_2.isUniversalSelector(s) ||
                selectors_2.isVueSpecialPseudo(s));
            for (let index = 0; index < selectorNodes.length; index++) {
                const selectorNode = selectorNodes[index];
                targetsQueryContext = targetsQueryContext.queryStep(selectorNode);
                if (!targetsQueryContext.elements.length) {
                    report(selectorNodes.slice(0, index + 1));
                    break;
                }
            }
        }
        return {
            "Program:exit"() {
                const queryContext = query_1.createQueryContext(context, options_1.parseQueryOptions(context.options[0]));
                for (const style of styles) {
                    for (const scopedSelector of getScopedSelectors(style)) {
                        verifySelector(queryContext, scopedSelector);
                    }
                }
            },
        };
    },
};
